﻿using System;
using System.ComponentModel;
using System.Windows.Input;
using System.Windows.Threading;

namespace ComboBoxSelectionCancel
{
    class MainViewModel : INotifyPropertyChanged
    {
        public MainViewModel()
        {
            ForceCommand = new DelegateCommand(OnForce);
            ClearLogCommand = new DelegateCommand(OnClearLog);
            Log.LogWrite += new System.Action<string>(OnLogWrite);
            ForceText = "March";
        }

        private readonly string[] _ItemsSource = new[] { "January", "February", "March" };
        public string[] ItemsSource
        {
            get { return _ItemsSource; }
        }

        public string ForceText { get; set; }

        private bool _AreUpdatesIgnored;
        public bool AreUpdatesIgnored
        {
            get { return _AreUpdatesIgnored; }
            set { _AreUpdatesIgnored = value; RaisePropetryChanged("AreUpdatesIgnored", false); }
        }

        private bool _ThrowExceptionOnUpdate;
        public bool ThrowExceptionOnUpdate
        {
            get { return _ThrowExceptionOnUpdate; }
            set { _ThrowExceptionOnUpdate = value; RaisePropetryChanged("ThrowExceptionOnUpdate", false); }
        }

        private bool _UseBeginInvoke;
        public bool UseBeginInvoke
        {
            get { return _UseBeginInvoke; }
            set { _UseBeginInvoke = value; RaisePropetryChanged("UseBeginInvoke", false); }
        }

        private string _SelectedItem;
        public string SelectedItem
        {
            get 
            {
                Log.Write("MainViewModel.get_SelectedItem() returns '" + _SelectedItem + "'");
                return _SelectedItem;  
            }
            set 
            {
                Log.Write("MainViewModel.set_SelectedItem('" + value + "')");

                if (ThrowExceptionOnUpdate)
                {
                    Log.Write("Throwing exception");
                    throw new InvalidOperationException();

                }

                if (AreUpdatesIgnored)
                {
                    Log.Write("Passed value ignored, MainViewModel.SelectedItem is still '" + _SelectedItem + "'");
                }
                else
                {
                    _SelectedItem = value;
                    Log.Write("MainViewModel.SelectedItem has been set to '" + _SelectedItem + "'");
                }

                if (UseBeginInvoke)
                {
                    Action deferred = () => { RaisePropetryChanged("SelectedItem", true); };
                    Dispatcher.CurrentDispatcher.BeginInvoke(deferred);
                }

                RaisePropetryChanged("SelectedItem", true);
                RaisePropetryChanged("SelectedItemForTextBlockDisplay", false);

            }
        }

        public string SelectedItemForTextBlockDisplay
        {
            get { return _SelectedItem; }
        }

        private string _LogText;
        public string LogText
        {
            get { return _LogText;  }
            set { _LogText = value; RaisePropetryChanged("LogText", false); }
        }

        public ICommand ForceCommand { get; private set; }
        public ICommand ClearLogCommand { get; private set; }

        private void OnForce()
        {
            SelectedItem = ForceText;
        }

        private void OnLogWrite(string obj)
        {
            LogText += obj + Environment.NewLine;
        }

        private void OnClearLog()
        {
            LogText = string.Empty;
        }

        private void RaisePropetryChanged(string name, bool logIt)
        {
            if (logIt)
            {
                Log.Write("Property changed '" + name + "'");
            }
            PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        public event PropertyChangedEventHandler PropertyChanged = delegate {};
    }
}
